@@ -1,6 +1,7 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE Rank2Types #-}
 {-# LANGUAGE TypeFamilies #-}
+{-# LANGUAGE TypeOperators #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE ScopedTypeVariables #-}
@@ -49,7 +50,6 @@ import Data.Functor.Bind
 import Data.Int
 import Data.Profunctor.Closed
 import Data.Profunctor.Rep
-import Control.Lens.Internal.Coerce
 
 -- $setup
 -- >>> :set -XNoOverloadedStrings
@@ -152,7 +152,7 @@ instance Profunctor (Indexed i) where
   {-# INLINE rmap #-}
   (.#) ibc _ = coerce ibc
   {-# INLINE (.#) #-}
-  (#.) _ = coerce'
+  (#.) _ = coerce
   {-# INLINE (#.) #-}
 
 instance Closed (Indexed i) where
@@ -282,10 +282,12 @@ instance Monoid (f a) => Monoid (Indexing f a) where
     mempty = Indexing $ \i -> (i, mempty)
     {-# INLINE mempty #-}
 
+#if !(MIN_VERSION_base(4,11,0))
     mappend (Indexing mx) (Indexing my) = Indexing $ \i -> case mx i of
       (j, x) -> case my j of
          ~(k, y) -> (k, mappend x y)
     {-# INLINE mappend #-}
+#endif
 
 -- | Transform a 'Control.Lens.Traversal.Traversal' into an 'Control.Lens.Traversal.IndexedTraversal' or
 -- a 'Control.Lens.Fold.Fold' into an 'Control.Lens.Fold.IndexedFold', etc.
